Querying — SQLAlchemy 0.6 Documentation

# SQLAlchemy 0.6 Documentation

Search:
Release: 0.6.9 | Release Date: May 5, 2012

# Querying

This section provides API documentation for the `Query` object and related constructs.

For an in-depth introduction to querying with the SQLAlchemy ORM, please see the Object Relational Tutorial.

## The Query Object

`Query` is produced in terms of a given `Session`, using the `query()` function:

    q = session.query(SomeMappedClass)

Following is the full interface for the `Query` object.

class `sqlalchemy.orm.query.` `Query`(entities, session=None)

  ORM-level SQL construction object.

  `Query` is the source of all SELECT statements generated by the ORM, both those formulated by end-user query operations as well as by high level internal operations such as related collection loading. It features a generative interface whereby successive calls return a new `Query` object, a copy of the former with additional criteria and options associated with it.

  `Query` objects are normally initially generated using the `query()` method of `Session`. For a full walkthrough of `Query` usage, see the Object Relational Tutorial.

  `add_column`(column)

    Add a column expression to the list of result columns to be returned.

    Pending deprecation: `add_column()` will be superseded by `add_columns()`.

  `add_columns`(*column)

    Add one or more column expressions to the list of result columns to be returned.

  `add_entity`(entity, alias=None)

    add a mapped entity to the list of result columns to be returned.

  `all`()

    Return the results represented by this `Query` as a list.

    This results in an execution of the underlying query.

  `as_scalar`()

    Return the full SELECT statement represented by this `Query`, converted to a scalar subquery.

    Analogous to `sqlalchemy.sql._SelectBaseMixin.as_scalar()`.

    New in 0.6.5.

  `autoflush`(setting)

    Return a Query with a specific ‘autoflush’ setting.

    Note that a Session with autoflush=False will not autoflush, even if this flag is set to True at the Query level. Therefore this flag is usually used only to disable autoflush for a specific Query.

  `column_descriptions`

    Return metadata about the columns which would be returned by this `Query`.

    Format is a list of dictionaries:

        user_alias = aliased(User, name='user2')
        q = sess.query(User, User.id, user_alias)

        # this expression:
        q.column_descriptions

        # would return:
        [
            {
                'name':'User',
                'type':User,
                'aliased':False,
                'expr':User,
            },
            {
                'name':'id',
                'type':Integer(),
                'aliased':False,
                'expr':User.id,
            },
            {
                'name':'user2',
                'type':User,
                'aliased':True,
                'expr':user_alias
            }
        ]

  `correlate`(*args)

    Return a `Query` construct which will correlate the given FROM clauses to that of an enclosing `Query` or `select()`.

    The method here accepts mapped classes, `aliased()` constructs, and `mapper()` constructs as arguments, which are resolved into expression constructs, in addition to appropriate expression constructs.

    The correlation arguments are ultimately passed to `Select.correlate()` after coercion to expression constructs.

    The correlation arguments take effect in such cases as when `Query.from_self()` is used, or when a subquery as returned by `Query.subquery()` is embedded in another `select()` construct.

  `count`()

    Return a count of rows this Query would return.

    For simple entity queries, count() issues a SELECT COUNT, and will specifically count the primary key column of the first entity only. If the query uses LIMIT, OFFSET, or DISTINCT, count() will wrap the statement generated by this Query in a subquery, from which a SELECT COUNT is issued, so that the contract of “how many rows would be returned?” is honored.

    For queries that request specific columns or expressions, count() again makes no assumptions about those expressions and will wrap everything in a subquery. Therefore, `Query.count()` is usually not what you want in this case. To count specific columns, often in conjunction with GROUP BY, use `func.count()` as an individual column expression instead of `Query.count()`. See the ORM tutorial for an example.

  `delete`(synchronize_session='evaluate')

    Perform a bulk delete query.

    Deletes rows matched by this query from the database.

    Parameters: | synchronize_session – chooses the strategy for the removal of matched objects from the session. Valid values are: False - don’t synchronize the session. This option is the most efficient and is reliable once the session is expired, which typically occurs after a commit(), or explicitly using expire_all(). Before the expiration, objects may still remain in the session which were in fact deleted which can lead to confusing results if they are accessed via get() or already loaded collections. ‘fetch’ - performs a select query before the delete to find objects that are matched by the delete query and need to be removed from the session. Matched objects are removed from the session. ‘evaluate’ - Evaluate the query’s criteria in Python straight on the objects in the session. If evaluation of the criteria isn’t implemented, an error is raised. In that case you probably want to use the ‘fetch’ strategy as a fallback. The expression evaluator currently doesn’t account for differing string collations between the database and Python.

    Returns the number of rows deleted, excluding any cascades.

    The method does not offer in-Python cascading of relationships - it is assumed that ON DELETE CASCADE is configured for any foreign key references which require it. The Session needs to be expired (occurs automatically after commit(), or call expire_all()) in order for the state of dependent objects subject to delete or delete-orphan cascade to be correctly represented.

    Also, the `before_delete()` and `after_delete()` `MapperExtension` methods are not called from this method. For a delete hook here, use the `SessionExtension.after_bulk_delete()` event hook.

  `distinct`()

    Apply a `DISTINCT` to the query and return the newly resulting `Query`.

  `enable_assertions`(value)

    Control whether assertions are generated.

    When set to False, the returned Query will not assert its state before certain operations, including that LIMIT/OFFSET has not been applied when filter() is called, no criterion exists when get() is called, and no “from_statement()” exists when filter()/order_by()/group_by() etc. is called. This more permissive mode is used by custom Query subclasses to specify criterion or other modifiers outside of the usual usage patterns.

    Care should be taken to ensure that the usage pattern is even possible. A statement applied by from_statement() will override any criterion set by filter() or order_by(), for example.

  `enable_eagerloads`(value)

    Control whether or not eager joins and subqueries are rendered.

    When set to False, the returned Query will not render eager joins regardless of `joinedload()`, `subqueryload()` options or mapper-level `lazy='joined'`/`lazy='subquery'` configurations.

    This is used primarily when nesting the Query’s statement into a subquery or other selectable.

  `except_`(*q)

    Produce an EXCEPT of this Query against one or more queries.

    Works the same way as `union()`. See that method for usage examples.

  `except_all`(*q)

    Produce an EXCEPT ALL of this Query against one or more queries.

    Works the same way as `union()`. See that method for usage examples.

  `execution_options`(**kwargs)

    Set non-SQL options which take effect during execution.

    The options are the same as those accepted by `sqlalchemy.sql.expression.Executable.execution_options()`.

    Note that the `stream_results` execution option is enabled automatically if the `yield_per()` method is used.

  `filter`(criterion)

    apply the given filtering criterion to the query and return the newly resulting `Query`

    the criterion is any sql.ClauseElement applicable to the WHERE clause of a select.

  `filter_by`(**kwargs)

    apply the given filtering criterion to the query and return the newly resulting `Query`.

  `first`()

    Return the first result of this `Query` or None if the result doesn’t contain any row.

    first() applies a limit of one within the generated SQL, so that only one primary entity row is generated on the server side (note this may consist of multiple result rows if join-loaded collections are present).

    Calling `first()` results in an execution of the underlying query.

  `from_self`(*entities)

    return a Query that selects from this Query’s SELECT statement.

    *entities - optional list of entities which will replace those being selected.

  `from_statement`(statement)

    Execute the given SELECT statement and return results.

    This method bypasses all internal statement compilation, and the statement is executed without modification.

    The statement argument is either a string, a `select()` construct, or a `text()` construct, and should return the set of columns appropriate to the entity class represented by this `Query`.

  `get`(ident)

    Return an instance based on the given primary key identifier, or `None` if not found.

    E.g.:

        my_user = session.query(User).get(5)

        some_object = session.query(VersionedFoo).get((5, 10))

    `get()` is special in that it provides direct access to the identity map of the owning `Session`. If the given primary key identifier is present in the local identity map, the object is returned directly from this collection and no SQL is emitted, unless the object has been marked fully expired. If not present, a SELECT is performed in order to locate the object.

    `get()` also will perform a check if the object is present in the identity map and marked as expired - a SELECT is emitted to refresh the object as well as to ensure that the row is still present. If not, `ObjectDeletedError` is raised.

    `get()` is only used to return a single mapped instance, not multiple instances or individual column constructs, and strictly on a single primary key value. The originating `Query` must be constructed in this way, i.e. against a single mapped entity, with no additional filtering criterion. Loading options via `options()` may be applied however, and will be used if the object is not yet locally present.

    A lazy-loading, many-to-one attribute configured by `relationship()`, using a simple foreign-key-to-primary-key criterion, will also use an operation equivalent to `get()` in order to retrieve the target value from the local identity map before querying the database. See Relationship Loading Techniques for further details on relationship loading.

    Parameters: | ident – A scalar or tuple value representing the primary key. For a composite primary key, the order of identifiers corresponds in most cases to that of the mapped `Table` object’s primary key columns. For a `mapper()` that was given the `primary key` argument during construction, the order of identifiers corresponds to the elements present in this collection.
    Returns: | The object instance, or `None`.

  `group_by`(*criterion)

    apply one or more GROUP BY criterion to the query and return the newly resulting `Query`

  `having`(criterion)

    apply a HAVING criterion to the query and return the newly resulting `Query`.

  `instances`(cursor, _Query__context=None)

    Given a ResultProxy cursor as returned by connection.execute(), return an ORM result as an iterator.

    e.g.:

        result = engine.execute("select * from users")
        for u in session.query(User).instances(result):
            print u

  `intersect`(*q)

    Produce an INTERSECT of this Query against one or more queries.

    Works the same way as `union()`. See that method for usage examples.

  `intersect_all`(*q)

    Produce an INTERSECT ALL of this Query against one or more queries.

    Works the same way as `union()`. See that method for usage examples.

  `join`(*props, **kwargs)

    Create a join against this `Query` object’s criterion and apply generatively, returning the newly resulting `Query`.

    Each element in *props may be:

    > - a string property name, i.e. “rooms”. This will join along the relationship of the same name from this Query’s “primary” mapper, if one is present.
    > - a class-mapped attribute, i.e. Houses.rooms. This will create a join from “Houses” table to that of the “rooms” relationship.
    > - a 2-tuple containing a target class or selectable, and an “ON” clause. The ON clause can be the property name/ attribute like above, or a SQL expression.

    e.g.:

        # join along string attribute names
        session.query(Company).join('employees')
        session.query(Company).join('employees', 'tasks')

        # join the Person entity to an alias of itself,
        # along the "friends" relationship
        PAlias = aliased(Person)
        session.query(Person).join((Palias, Person.friends))

        # join from Houses to the "rooms" attribute on the
        # "Colonials" subclass of Houses, then join to the
        # "closets" relationship on Room
        session.query(Houses).join(Colonials.rooms, Room.closets)

        # join from Company entities to the "employees" collection,
        # using "people JOIN engineers" as the target.  Then join
        # to the "computers" collection on the Engineer entity.
        session.query(Company).\
                    join((people.join(engineers), 'employees'),
                    Engineer.computers)

        # join from Articles to Keywords, using the "keywords" attribute.
        # assume this is a many-to-many relationship.
        session.query(Article).join(Article.keywords)

        # same thing, but spelled out entirely explicitly
        # including the association table.
        session.query(Article).join(
            (article_keywords,
            Articles.id==article_keywords.c.article_id),
            (Keyword, Keyword.id==article_keywords.c.keyword_id)
            )

    **kwargs include:

    > aliased - when joining, create anonymous aliases of each table. This is used for self-referential joins or multiple joins to the same table. Consider usage of the aliased(SomeClass) construct as a more explicit approach to this.

    > from_joinpoint - when joins are specified using string property names, locate the property from the mapper found in the most recent previous join() call, instead of from the root entity.

    See also Querying with Joins in the ORM tutorial.

  `label`(name)

    Return the full SELECT statement represented by this `Query`, converted to a scalar subquery with a label of the given name.

    Analogous to `sqlalchemy.sql._SelectBaseMixin.label()`.

    New in 0.6.5.

  `limit`(limit)

    Apply a `LIMIT` to the query and return the newly resulting

    `Query`.

  `logger` = <logging.Logger object at 0x1023ba4d0>
  `merge_result`(iterator, load=True)

    Merge a result into this Query’s Session.

    Given an iterator returned by a Query of the same structure as this one, return an identical iterator of results, with all mapped instances merged into the session using Session.merge(). This is an optimized method which will merge all mapped instances, preserving the structure of the result rows and unmapped columns with less method overhead than that of calling Session.merge() explicitly for each value.

    The structure of the results is determined based on the column list of this Query - if these do not correspond, unchecked errors will occur.

    The ‘load’ argument is the same as that of Session.merge().

  `offset`(offset)

    Apply an `OFFSET` to the query and return the newly resulting `Query`.

  `one`()

    Return exactly one result or raise an exception.

    Raises `sqlalchemy.orm.exc.NoResultFound` if the query selects no rows. Raises `sqlalchemy.orm.exc.MultipleResultsFound` if multiple object identities are returned, or if multiple rows are returned for a query that does not return object identities.

    Note that an entity query, that is, one which selects one or more mapped classes as opposed to individual column attributes, may ultimately represent many rows but only one row of unique entity or entities - this is a successful result for one().

    Calling `one()` results in an execution of the underlying query. As of 0.6, `one()` fully fetches all results instead of applying any kind of limit, so that the “unique”-ing of entities does not conceal multiple object identities.

  `options`(*args)

    Return a new Query object, applying the given list of mapper options.

    Most supplied options regard changing how column- and relationship-mapped attributes are loaded. See the sections Deferred Column Loading and Relationship Loading Techniques for reference documentation.

  `order_by`(*criterion)

    apply one or more ORDER BY criterion to the query and return the newly resulting `Query`

    All existing ORDER BY settings can be suppressed by passing `None` - this will suppress any ORDER BY configured on mappers as well.

    Alternatively, an existing ORDER BY setting on the Query object can be entirely cancelled by passing `False` as the value - use this before calling methods where an ORDER BY is invalid.

  `outerjoin`(*props, **kwargs)

    Create a left outer join against this `Query` object’s criterion and apply generatively, returning the newly resulting `Query`.

    Usage is the same as the `join()` method.

  `params`(*args, **kwargs)

    add values for bind parameters which may have been specified in filter().

    parameters may be specified using **kwargs, or optionally a single dictionary as the first positional argument. The reason for both is that **kwargs is convenient, however some parameter dictionaries contain unicode keys in which case **kwargs cannot be used.

  `populate_existing`()

    Return a `Query` that will expire and refresh all instances as they are loaded, or reused from the current `Session`.

    `populate_existing()` does not improve behavior when the ORM is used normally - the `Session` object’s usual behavior of maintaining a transaction and expiring all attributes after rollback or commit handles object state automatically. This method is not intended for general use.

  `reset_joinpoint`()

    return a new Query reset the ‘joinpoint’ of this Query reset back to the starting mapper. Subsequent generative calls will be constructed from the new joinpoint.

    Note that each call to join() or outerjoin() also starts from the root.

  `scalar`()

    Return the first element of the first result or None if no rows present. If multiple rows are returned, raises MultipleResultsFound.

        >>> session.query(Item).scalar()
        <Item>
        >>> session.query(Item.id).scalar()
        1
        >>> session.query(Item.id).filter(Item.id < 0).scalar()
        None
        >>> session.query(Item.id, Item.name).scalar()
        1
        >>> session.query(func.count(Parent.id)).scalar()
        20

    This results in an execution of the underlying query.

  `select_from`(*from_obj)

    Set the FROM clause of this `Query` explicitly.

    Sending a mapped class or entity here effectively replaces the “left edge” of any calls to `Query.join()`, when no joinpoint is otherwise established - usually, the default “join point” is the leftmost entity in the `Query` object’s list of entities to be selected.

    Mapped entities or plain `Table` or other selectables can be sent here which will form the default FROM clause.

  `slice`(start, stop)

    apply LIMIT/OFFSET to the `Query` based on a ” “range and return the newly resulting `Query`.

  `statement`

    The full SELECT statement represented by this Query.

    The statement by default will not have disambiguating labels applied to the construct unless with_labels(True) is called first.

  `subquery`(name=None)

    return the full SELECT statement represented by this `Query`, embedded within an `Alias`.

    Eager JOIN generation within the query is disabled.

    The statement will not have disambiguating labels applied to the list of selected columns unless the `Query.with_labels()` method is used to generate a new `Query` with the option enabled.

    Parameters: | name – string name to be assigned as the alias; this is passed through to `FromClause.alias()`. If `None`, a name will be deterministically generated at compile time.

  `union`(*q)

    Produce a UNION of this Query against one or more queries.

    e.g.:

        q1 = sess.query(SomeClass).filter(SomeClass.foo=='bar')
        q2 = sess.query(SomeClass).filter(SomeClass.bar=='foo')

        q3 = q1.union(q2)

    The method accepts multiple Query objects so as to control the level of nesting. A series of `union()` calls such as:

        x.union(y).union(z).all()

    will nest on each `union()`, and produces:

        SELECT * FROM (SELECT * FROM (SELECT * FROM X UNION
                        SELECT * FROM y) UNION SELECT * FROM Z)

    Whereas:

        x.union(y, z).all()

    produces:

        SELECT * FROM (SELECT * FROM X UNION SELECT * FROM y UNION
                        SELECT * FROM Z)

  `union_all`(*q)

    Produce a UNION ALL of this Query against one or more queries.

    Works the same way as `union()`. See that method for usage examples.

  `update`(values, synchronize_session='evaluate')

    Perform a bulk update query.

    Updates rows matched by this query in the database.

    Parameters: | values – a dictionary with attributes names as keys and literal values or sql expressions as values. synchronize_session – chooses the strategy to update the attributes on objects in the session. Valid values are: False - don’t synchronize the session. This option is the most efficient and is reliable once the session is expired, which typically occurs after a commit(), or explicitly using expire_all(). Before the expiration, updated objects may still remain in the session with stale values on their attributes, which can lead to confusing results. ‘fetch’ - performs a select query before the update to find objects that are matched by the update query. The updated attributes are expired on matched objects. ‘evaluate’ - Evaluate the Query’s criteria in Python straight on the objects in the session. If evaluation of the criteria isn’t implemented, an exception is raised. The expression evaluator currently doesn’t account for differing string collations between the database and Python.

    Returns the number of rows matched by the update.

    The method does not offer in-Python cascading of relationships - it is assumed that ON UPDATE CASCADE is configured for any foreign key references which require it.

    The Session needs to be expired (occurs automatically after commit(), or call expire_all()) in order for the state of dependent objects subject foreign key cascade to be correctly represented.

    Also, the `before_update()` and `after_update()` `MapperExtension` methods are not called from this method. For an update hook here, use the `SessionExtension.after_bulk_update()` event hook.

  `value`(column)

    Return a scalar result corresponding to the given column expression.

  `values`(*columns)

    Return an iterator yielding result tuples corresponding to the given list of columns

  `whereclause`

    A readonly attribute which returns the current WHERE criterion for this Query.

    This returned value is a SQL expression construct, or `None` if no criterion has been established.

  `with_entities`(*entities)

    Return a new `Query` replacing the SELECT list with the given entities.

    e.g.:

        # Users, filtered on some arbitrary criterion
        # and then ordered by related email address
        q = session.query(User).\
                    join(User.address).\
                    filter(User.name.like('%ed%')).\
                    order_by(Address.email)

        # given *only* User.id==5, Address.email, and 'q', what
        # would the *next* User in the result be ?
        subq = q.with_entities(Address.email).\
                    order_by(None).\
                    filter(User.id==5).\
                    subquery()
        q = q.join((subq, subq.c.email < Address.email)).\
                    limit(1)

    New in 0.6.5.

  `with_hint`(selectable, text, dialect_name='*')

    Add an indexing hint for the given entity or selectable to this `Query`.

    Functionality is passed straight through to `with_hint()`, with the addition that `selectable` can be a `Table`, `Alias`, or ORM entity / mapped class /etc.

  `with_labels`()

    Apply column labels to the return value of Query.statement.

    Indicates that this Query’s statement accessor should return a SELECT statement that applies labels to all columns in the form <tablename>_<columnname>; this is commonly used to disambiguate columns from multiple tables which have the same name.

    When the Query actually issues SQL to load rows, it always uses column labeling.

  `with_lockmode`(mode)

    Return a new Query object with the specified locking mode.

    Parameters: | mode – a string representing the desired locking mode. A corresponding value is passed to the `for_update` parameter of `select()` when the query is executed. Valid values are: `'update'` - passes `for_update=True`, which translates to `FOR UPDATE` (standard SQL, supported by most dialects) `'update_nowait'` - passes `for_update='nowait'`, which translates to `FOR UPDATE NOWAIT` (supported by Oracle) `'read'` - passes `for_update='read'`, which translates to `LOCK IN SHARE MODE` (supported by MySQL).

  `with_parent`(instance, property=None)

    Add filtering criterion that relates the given instance to a child object or collection, using its attribute state as well as an established `relationship()` configuration.

    The method uses the `with_parent()` function to generate the clause, the result of which is passed to `Query.filter()`.

    Parameters are the same as `with_parent()`, with the exception that the given property can be None, in which case a search is performed against this `Query` object’s target mapper.

  `with_polymorphic`(cls_or_mappers, selectable=None, discriminator=None)

    Load columns for descendant mappers of this Query’s mapper.

    Using this method will ensure that each descendant mapper’s tables are included in the FROM clause, and will allow filter() criterion to be used against those tables. The resulting instances will also have those columns already loaded so that no “post fetch” of those columns will be required.

    Parameters: | cls_or_mappers – a single class or mapper, or list of class/mappers, which inherit from this Query’s mapper. Alternatively, it may also be the string `'*'`, in which case all descending mappers will be added to the FROM clause. selectable – a table or select() statement that will be used in place of the generated FROM clause. This argument is required if any of the desired mappers use concrete table inheritance, since SQLAlchemy currently cannot generate UNIONs among tables automatically. If used, the `selectable` argument must represent the full set of tables and columns mapped by every desired mapper. Otherwise, the unaccounted mapped columns will result in their table being appended directly to the FROM clause which will usually lead to incorrect results. discriminator – a column to be used as the “discriminator” column for the given selectable. If not given, the polymorphic_on attribute of the mapper will be used, if any. This is useful for mappers that don’t have polymorphic loading behavior by default, such as concrete table mappers.

  `yield_per`(count)

    Yield only `count` rows at a time.

    WARNING: use this method with caution; if the same instance is present in more than one batch of rows, end-user changes to attributes will be overwritten.

    In particular, it’s usually impossible to use this setting with eagerly loaded collections (i.e. any lazy=’joined’ or ‘subquery’) since those collections will be cleared for a new load when encountered in a subsequent result batch. In the case of ‘subquery’ loading, the full result for all rows is fetched which generally defeats the purpose of `yield_per()`.

    Also note that many DBAPIs do not “stream” results, pre-buffering all rows before making them available, including mysql-python and psycopg2. `yield_per()` will also set the `stream_results` execution option to `True`, which currently is only understood by psycopg2 and causes server side cursors to be used.

## ORM-Specific Query Constructs

class `sqlalchemy.orm.` `aliased`

The public name of the `AliasedClass` class.

class `sqlalchemy.orm.util.` `AliasedClass`(cls, alias=None, name=None)

  Represents an “aliased” form of a mapped class for usage with Query.

  The ORM equivalent of a `sqlalchemy.sql.expression.alias()` construct, this object mimics the mapped class using a __getattr__ scheme and maintains a reference to a real `Alias` object.

  Usage is via the `aliased()` synonym:

      # find all pairs of users with the same name
      user_alias = aliased(User)
      session.query(User, user_alias).\
                      join((user_alias, User.id > user_alias.id)).\
                      filter(User.name==user_alias.name)

`sqlalchemy.orm.` `join`(left, right, onclause=None, isouter=False, join_to_left=True)

  Produce an inner join between left and right clauses.

  In addition to the interface provided by `join()`, left and right may be mapped classes or AliasedClass instances. The onclause may be a string name of a relationship(), or a class-bound descriptor representing a relationship.

  join_to_left indicates to attempt aliasing the ON clause, in whatever form it is passed, to the selectable passed as the left side. If False, the onclause is used as is.

`sqlalchemy.orm.` `outerjoin`(left, right, onclause=None, join_to_left=True)

  Produce a left outer join between left and right clauses.

  In addition to the interface provided by `outerjoin()`, left and right may be mapped classes or AliasedClass instances. The onclause may be a string name of a relationship(), or a class-bound descriptor representing a relationship.

`sqlalchemy.orm.` `with_parent`(instance, prop)

  Create filtering criterion that relates this query’s primary entity to the given related instance, using established `relationship()` configuration.

  The SQL rendered is the same as that rendered when a lazy loader would fire off from the given parent on that attribute, meaning that the appropriate state is taken from the parent object in Python without the need to render joins to the parent table in the rendered statement.

  As of 0.6.4, this method accepts parent instances in all persistence states, including transient, persistent, and detached. Only the requisite primary key/foreign key attributes need to be populated. Previous versions didn’t work with transient instances.

  Parameters: | instance – An instance which has some `relationship()`. property – String property name, or class-bound attribute, which indicates what relationship from the instance should be used to reconcile the parent/child relationship.